@@ -17,7 +17,7 @@ class QuerySet(models.QuerySet):
             return self
 
         if len(items) == 1:
-            return self.filter(**{field: items[0]})
+            return self.filter(**{field: tuple(items)[0]})
 
         return self.filter(**{f'{field}__in': items})
 
