@@ -1,4 +1,6 @@
 from typing import Any, List, Optional, Sequence, TypeVar
+from django.db import models
+from django.contrib.postgres.search import SearchVector, SearchQuery, SearchRank
 
 from wcd_geo_db.const import DivisionLevel
 from wcd_geo_db.modules.code_seeker.query import CodeSeekerQuerySet
@@ -7,7 +9,7 @@ from pxd_tree.hierarchy import TreeQuerySet
 from ..dtos import DivisionDTO, GeometryDTO
 from .base import QuerySet
 from .geometry import WithGeometryQuerySet
-from .search import SearchQuerySet
+from .search import SearchQueryParam, SearchQuerySet
 
 
 __all__ = 'DivisionsQuerySet',
@@ -42,6 +44,32 @@ class DivisionsQuerySet(
     SearchQuerySet,
     QuerySet
 ):
+    SEARCH_QUERY_MIN_LENGTH: int = 2
+    SEARCH_QUERY_RANK_WEIGHTS: Sequence[float] = [0.2, 0.4, 0.8, 1]
+
+    def search(self: QT, query: SearchQueryParam) -> QT:
+        q = query.get('query')
+
+        if not q or len(q) < self.SEARCH_QUERY_MIN_LENGTH:
+            return self
+
+        vector = (
+            SearchVector('name', weight='A')
+            +
+            SearchVector('synonyms', weight='B')
+        )
+        sql_query = SearchQuery(q, search_type='websearch')
+        rank = SearchRank(
+            vector, sql_query, weights=self.SEARCH_QUERY_RANK_WEIGHTS
+        )
+
+        return (
+            self
+            .annotate(search=vector, rank=rank)
+            .filter(search=sql_query)
+            .order_by('-rank')
+        )
+
     def as_dtos(self) -> List[DivisionDTO]:
         return [as_dto(values) for values in self.values(*VALUES_NAMES)]
 
